@@ -62,6 +62,12 @@
 #undef SIZEOF_INT
 #undef SIZEOF_VOID_P
 
+#undef G_VA_COPY
+#undef G_VA_COPY_AS_ARRAY
+#undef G_HAVE___INLINE
+#undef G_HAVE___INLINE__
+#undef G_HAVE_INLINE
+
 #undef GLIB_MAJOR_VERSION
 #undef GLIB_MINOR_VERSION
 #undef GLIB_MICRO_VERSION
@@ -71,6 +77,8 @@
 /* #undef PACKAGE */
 /* #undef VERSION */
 
+
+
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
