@@ -967,14 +967,14 @@ g_strcasecmp (const gchar *s1,
 #endif
 }
 
-void
+gchar*
 g_strdelimit (gchar	  *string,
 	      const gchar *delimiters,
 	      gchar	   new_delim)
 {
   register gchar *c;
   
-  g_return_if_fail (string != NULL);
+  g_return_val_if_fail (string != NULL, NULL);
   
   if (!delimiters)
     delimiters = G_STR_DELIMITERS;
@@ -984,4 +984,192 @@ g_strdelimit (gchar	  *string,
       if (strchr (delimiters, *c))
 	*c = new_delim;
     }
+
+  return string;
+}
+
+/* blame Elliot for these next five routines */
+gchar*
+g_strchug (gchar *string)
+{
+  gchar *start;
+
+  g_return_val_if_fail (string != NULL, NULL);
+
+  for (start = string; *start && isspace (*start); start++)
+    ;
+
+  strcpy (string, start);
+
+  return string;
+}
+
+gchar*
+g_strchomp (gchar *string)
+{
+  gchar *s;
+
+  g_return_val_if_fail (string != NULL, NULL);
+
+  if (!*string)
+    return string;
+
+  for (s = string + strlen (string) - 1; s >= string && isspace (*s); s--)
+    *s = '\0';
+
+  return string;
+}
+
+gchar**
+g_strsplit (const gchar *string,
+	    const gchar *delimiter,
+	    gint         max_tokens)
+{
+  GSList *string_list = NULL, *slist;
+  gchar **str_array, *s;
+  guint i, n = 1;
+
+  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (delimiter != NULL, NULL);
+
+  if (max_tokens < 1)
+    max_tokens = G_MAXINT;
+
+  s = strstr (string, delimiter);
+  if (s)
+    {
+      guint delimiter_len = strlen (delimiter);
+      
+      do
+	{
+	  guint len;
+	  gchar *new_string;
+	  
+	  len = s - string;
+	  new_string = g_new (gchar, len + 1);
+	  strncpy (new_string, string, len);
+	  new_string[len] = 0;
+	  string_list = g_slist_prepend (string_list, new_string);
+	  n++;
+	  string = s + delimiter_len;
+	  s = strstr (string, delimiter);
+	}
+      while (--max_tokens && s);
+    }
+  if (*string)
+    {
+      n++;
+      string_list = g_slist_prepend (string_list, g_strdup (string));
+    }
+  
+  str_array = g_new (gchar*, n);
+
+  i = n - 1;
+
+  str_array[i--] = NULL;
+  for (slist = string_list; slist; slist = slist->next)
+    str_array[i--] = slist->data;
+
+  g_slist_free (string_list);
+
+  return str_array;
+}
+
+void
+g_strfreev (gchar **str_array)
+{
+  if (str_array)
+    {
+      int i;
+
+      for(i = 0; str_array[i] != NULL; i++)
+	g_free(str_array[i]);
+
+      g_free (str_array);
+    }
+}
+
+gchar*
+g_strjoinv (const gchar  *separator,
+	    gchar       **str_array)
+{
+  gchar *string;
+  
+  g_return_val_if_fail (str_array != NULL, NULL);
+
+  if(separator == NULL)
+    separator = "";
+
+  if (*str_array)
+    {
+      guint i, len;
+      guint separator_len;
+
+      separator_len = strlen (separator);
+      len = 1 + strlen (str_array[0]);
+      for(i = 1; str_array[i] != NULL; i++)
+	len += separator_len + strlen(str_array[i]);
+      
+      string = g_new (gchar, len);
+      *string = 0;
+      strcat (string, *str_array);
+      for (i = 1; str_array[i] != NULL; i++)
+	{
+	  strcat (string, separator);
+	  strcat (string, str_array[i]);
+	}
+      }
+  else
+    string = g_strdup ("");
+
+  return string;
+}
+
+gchar*
+g_strjoin (const gchar  *separator,
+	   ...)
+{
+  gchar *string, *s;
+  va_list args;
+  guint len;
+  guint separator_len;
+
+  if(separator == NULL)
+    separator = "";
+
+  separator_len = strlen (separator);
+
+  va_start(args, separator);
+
+  s = va_arg(args, gchar *);
+
+  if(s) {
+    len = strlen(s) + 1;
+
+    while((s = va_arg(args, gchar*)))
+      {
+	len += separator_len + strlen(s);
+      }
+    va_end(args);
+
+    string = g_new (gchar, len);
+
+    va_start(args, separator);
+
+    *string = 0;
+    s = va_arg(args, gchar*);
+    strcat (string, s);
+
+    while((s = va_arg(args, gchar*)))
+      {
+	strcat(string, separator);
+	strcat(string, s);
+      }
+
+  } else
+    string = g_strdup("");
+
+  va_end(args);
+
+  return string;
 }
