@@ -87,6 +87,8 @@ g_tree_new (GCompareFunc key_compare_func)
 {
   GRealTree *rtree;
 
+  g_return_val_if_fail (key_compare_func != NULL, NULL);
+
   rtree = g_new (GRealTree, 1);
   rtree->root = NULL;
   rtree->key_compare = key_compare_func;
@@ -699,22 +701,23 @@ g_tree_node_check (GTreeNode *node)
   gint left_height;
   gint right_height;
   gint balance;
-
+  
   if (node)
     {
       left_height = 0;
       right_height = 0;
-
+      
       if (node->left)
 	left_height = g_tree_node_height (node->left);
       if (node->right)
 	right_height = g_tree_node_height (node->right);
-
+      
       balance = right_height - left_height;
       if (balance != node->balance)
-	g_print ("g_tree_node_check: failed: %d ( %d )\n",
-		 balance, node->balance);
-
+	g_log (g_log_domain_glib, G_LOG_LEVEL_INFO,
+	       "g_tree_node_check: failed: %d ( %d )\n",
+	       balance, node->balance);
+      
       if (node->left)
 	g_tree_node_check (node->left);
       if (node->right)
