@@ -16,6 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#undef G_LOG_DOMAIN
 
 #include <stdio.h>
 #include <string.h>
@@ -176,6 +177,30 @@ g_node_test (void)
     g_print ("ok\n");
 }
 
+gboolean
+my_hash_callback_remove (gpointer key,
+			 gpointer value,
+			 gpointer user_data)
+{
+  int *d = value;
+
+  if ((*d) % 2)
+    return TRUE;
+
+  return FALSE;
+}
+
+void
+my_hash_callback_remove_test (gpointer key,
+			      gpointer value,
+			      gpointer user_data)
+{
+  int *d = value;
+
+  if ((*d) % 2)
+    g_print ("bad!\n");
+}
+
 void
 my_hash_callback (gpointer key,
 		  gpointer value,
@@ -198,7 +223,7 @@ my_hash_compare (gconstpointer a,
   return *((const gint*) a) == *((const gint*) b);
 }
 
-gint 
+gint
 my_list_compare_one (gconstpointer a, gconstpointer b)
 {
   gint one = *((const gint*)a);
@@ -206,7 +231,7 @@ my_list_compare_one (gconstpointer a, gconstpointer b)
   return one-two;
 }
 
-gint 
+gint
 my_list_compare_two (gconstpointer a, gconstpointer b)
 {
   gint one = *((const gint*)a);
@@ -347,7 +372,7 @@ main (int   argc,
 
   g_list_free (list);
   list = NULL;
-  
+
   for (i = 0; i < 10; i++)
     list = g_list_insert_sorted (list, &morenums[i], my_list_compare_one);
 
@@ -362,10 +387,10 @@ main (int   argc,
       if (*((gint*) t->data) != i)
          g_error ("Sorted insert failed");
     }
-    
+
   g_list_free (list);
   list = NULL;
-  
+
   for (i = 0; i < 10; i++)
     list = g_list_insert_sorted (list, &morenums[i], my_list_compare_two);
 
@@ -380,7 +405,7 @@ main (int   argc,
       if (*((gint*) t->data) != (9 - i))
          g_error ("Sorted insert failed");
     }
-    
+
   g_list_free (list);
 
   g_print ("ok\n");
@@ -417,10 +442,10 @@ main (int   argc,
       if (*((gint*) st->data) != i)
          g_error ("Sorted insert failed");
     }
-     
+
   g_slist_free(slist);
   slist = NULL;
-   
+
   for (i = 0; i < 10; i++)
     slist = g_slist_insert_sorted (slist, &morenums[i], my_list_compare_two);
 
@@ -435,7 +460,7 @@ main (int   argc,
       if (*((gint*) st->data) != (9 - i))
          g_error("Sorted insert failed");
     }
-    
+
   g_slist_free(slist);
 
   g_print ("ok\n");
@@ -521,6 +546,19 @@ main (int   argc,
   for (i = 0; i < 10000; i++)
     g_hash_table_remove (hash_table, &array[i]);
 
+  for (i = 0; i < 10000; i++)
+    {
+      array[i] = i;
+      g_hash_table_insert (hash_table, &array[i], &array[i]);
+    }
+
+  if (g_hash_table_foreach_remove (hash_table, my_hash_callback_remove, NULL) != 5000 ||
+      g_hash_table_size (hash_table) != 5000)
+    g_print ("bad!\n");
+
+  g_hash_table_foreach (hash_table, my_hash_callback_remove_test, NULL);
+
+
   g_hash_table_destroy (hash_table);
 
   g_print ("ok\n");
@@ -554,9 +592,9 @@ main (int   argc,
 
   g_print ("checking arrays...");
 
-  garray = g_array_new (FALSE);
+  garray = g_array_new (FALSE, FALSE, sizeof (gint));
   for (i = 0; i < 10000; i++)
-    g_array_append_val (garray, gint, i);
+    g_array_append_val (garray, i);
 
   for (i = 0; i < 10000; i++)
     if (g_array_index (garray, gint, i) != i)
@@ -564,13 +602,13 @@ main (int   argc,
 
   g_array_free (garray, TRUE);
 
-  garray = g_array_new (FALSE);
-  for (i = 0; i < 10000; i++)
-    g_array_prepend_val (garray, gint, i);
+  garray = g_array_new (FALSE, FALSE, sizeof (gint));
+  for (i = 0; i < 100; i++)
+    g_array_prepend_val (garray, i);
 
-  for (i = 0; i < 10000; i++)
-    if (g_array_index (garray, gint, i) != (10000 - i - 1))
-      g_print ("uh oh: %d ( %d )\n", g_array_index (garray, gint, i), 10000 - i - 1);
+  for (i = 0; i < 100; i++)
+    if (g_array_index (garray, gint, i) != (100 - i - 1))
+      g_print ("uh oh: %d ( %d )\n", g_array_index (garray, gint, i), 100 - i - 1);
 
   g_array_free (garray, TRUE);
 
@@ -705,7 +743,7 @@ main (int   argc,
 
   gbarray = g_byte_array_new ();
   for (i = 0; i < 10000; i++)
-    g_byte_array_append (gbarray, "abcd", 4);
+    g_byte_array_append (gbarray, (guint8*) "abcd", 4);
 
   for (i = 0; i < 10000; i++)
     {
@@ -716,8 +754,13 @@ main (int   argc,
     }
 
   g_byte_array_free (gbarray, TRUE);
-
   g_print ("ok\n");
 
+  g_printerr ("g_log tests:");
+  g_warning ("harmless warning");
+  g_message ("the next warning is a test:");
+  string = NULL;
+  g_print (string);
+
   return 0;
 }
